
;**************************************************************************
;*                   Password Writer / Heslovacka                         *
;*                    utilizing I2C to USB Bridge                         *
;**************************************************************************
;**************************************************************************
;*          v 1.07 - 02.05.2020                                           *
;**************************************************************************
;*        Hardw.: PIC16F1829 used                                         *
;*        OSC.......: Internal osc. used     POWER.....:   5,0V extern.   *
;**************************************************************************

#define		SOW_out		TRISA,D'005'
#define		SOW_in		PORTA,D'005'


 INCLUDE "P16F1829.INC"

_U001			     EQU     B'00100110000100'		;see Page 48 of Datasheet
_U002			     EQU     B'01100011111111'		;see Page 49 of Datasheet

	__CONFIG	_CONFIG1, _U001
	__CONFIG	_CONFIG2, _U002

;==========================================================================
;       Variable Definition
;==========================================================================
TIMER1		EQU	H'20'		;delay routine
TIMER2		EQU	H'21'		; "	"	"

v_addr		EQU	H'22'
v_byte		EQU	H'23'
v_chip		EQU	H'24'

ow_buffer	EQU	H'25'
;==========================================================================
		ORG	0		;Reset vector address
		GOTO	init_main	;goto RESET routine when boot.
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
;              ++  RESET :  main boot routine  ++
;----------------------------------------------------------------------------
;----------------------------------------------------------------------------
init_main	nop
		clrf	BSR
		clrf	INTCON			;disable interrupts

		banksel	OPTION_REG
		movlw	B'00111111'
		movwf	OPTION_REG		;see datasheet Page 176
						;bit 7 = 0 --> Week pull-ups are enabled
		movlw	H'03'
		movwf	BSR			;select BANK 3
		clrf	ANSELA
		clrf	ANSELB
		clrf	ANSELC

		banksel	TRISA
		movlw	B'00000000'
		movwf	TRISA

		movlw	B'11111111'
		movwf	TRISC

		movlw	B'11111111'
		movwf	TRISB

		movf	OSCCON,0
		andlw	B'11110111'
		iorlw	B'01110000'
		movwf	OSCCON			;set 8MHz internal clock (bit 6:3 = 1110) - Datasheet Page 68

		banksel	WPUC
		bsf	WPUC,D'000'		;key for EEPROM 1010 000x
		bsf	WPUC,D'001'		;key for EEPROM 1010 001x
		bsf	WPUC,D'002'		;key for EEPROM 1010 010x
		bsf	WPUC,D'003'		;key for EEPROM 1010 011x
		bsf	WPUC,D'004'		;key for OneWire EEPROM

		bsf	WPUA,D'005'		;Pull-Up for OneWire Data-Line

		goto	main_prg
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;------------------------------------------------------------------------------
;                      Main Program
;------------------------------------------------------------------------------
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
;XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX
main_prg	clrf	BSR
		call	i2c_init

main_loop	bsf	LATA,D'000'
		call	dr2
		call	keys_process
		bcf	LATA,D'000'
		call	dr2
		call	keys_process
		goto	main_loop
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
keys_process	nop
		btfss	PORTC,D'000'
		call	send_pass_1
		btfss	PORTC,D'001'
		call	send_pass_2
		btfss	PORTC,D'002'
		call	send_pass_3
		btfss	PORTC,D'003'
		call	send_pass_4
		btfss	PORTC,D'004'
		call	send_pass_1w
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
send_pass_1	movlw	H'00'
		movwf	v_chip
		goto	send_pass_gen
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
send_pass_2	movlw	H'02'
		movwf	v_chip
		goto	send_pass_gen
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
send_pass_3	movlw	H'04'
		movwf	v_chip
		goto	send_pass_gen
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
send_pass_4	movlw	H'06'
		movwf	v_chip
		goto	send_pass_gen
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
send_pass_1w	call	send_pass_1we
		call	ow_reset
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
send_pass_1we	clrf	v_addr

		call	ow_reset
		movlw	H'CC'			;skip ROM
		movwf	ow_buffer
		call	ow_write
		movlw	H'F0'			;read memory
		movwf	ow_buffer
		call	ow_write
;---------------------------------------
		movf	v_addr,0		;EEPROM Address
		movwf	ow_buffer
		call	ow_write
		movlw	H'00'
		movwf	ow_buffer
		call	ow_write
;---------------------------------------
send_loop_1w	call	ow_read
		movf	ow_buffer,0
		movwf	v_byte
		xorlw	H'FF'
		btfsc	STATUS,Z
		return				;FFh received
		call	bridge_send
		incf	v_addr,1
		btfsc	STATUS,Z
		return				;256 bytes send already
		goto	send_loop_1w
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
send_pass_gen	bsf	LATA,D'000'
		clrf	v_addr

send_loop	call	ee_read_byte
		movf	v_byte,0
		xorlw	H'FF'
		btfsc	STATUS,Z
		return				;FFh received
		call	bridge_send
		incf	v_addr,1
		btfsc	STATUS,Z
		return				;256 bytes send already
		goto	send_loop
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ee_read_byte	call	i2c_start
		movlw	B'10100000'
		addwf	v_chip,0
		call	i2c_send
		movf	v_addr,0
		call	i2c_send
		call	i2c_stop
		call	i2c_start
		movlw	B'10100001'
		addwf	v_chip,0
		call	i2c_send
		call	i2c_receive
		call	i2c_not_ack
		call	i2c_stop
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
bridge_send	call	i2c_start
		movlw	B'01100000'		;I2C address of USB bridge
		call	i2c2usb
		movf	v_byte,0
		call	i2c2usb
		call	i2c_stop
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------
dr56		MOVLW   D'050'
		MOVWF   TIMER2
DEL_LOOPb1q	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOPb2q	DECFSZ  TIMER1,F
		GOTO    DEL_LOOPb2q
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOPb1q
		return
;-------------------------------------
i2c2usb		nop
		call	i2c_send
		call	dr56
		return
;-------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
; delay routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
d2		MOVLW   D'255'
		MOVWF   TIMER2
DEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

DEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    DEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    DEL_LOOP1
		return
;------------------------------------------------------------------------------
d2c		MOVLW   D'090'
		MOVWF   TIMER2
cDEL_LOOP1	MOVLW   D'255'
		MOVWF   TIMER1

cDEL_LOOP2	DECFSZ  TIMER1,F
		GOTO    cDEL_LOOP2
		DECFSZ  TIMER2,F
		GOTO    cDEL_LOOP1
		return
;------------------------------------------------------------------------------

;------------------------------------------------------------------------------
;------------------------------------------------------------------------------

;------------------------------------
dr2		call	d2
		call	d2
		call	d2
		return
;------------------------------------
dr4		call	dr2
		call	dr2
		call	dr2
		return
;-----------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;
; I2C communication routines
;
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx

;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;I2C
;set up I2C BUS for communication as a master
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
i2c_init	nop
		banksel	TRISB
		bsf	TRISB,D'007'	;SCL
		bsf	TRISB,D'005'	;SDA
		movlw	H'04'
		movwf	BSR		;select BANK 4

		movlw	B'00101000'		;see Datasheet Page 277
;			bit 3-0 SSPM<3:0>: Synchronous Serial Port Mode Select bits
;			1000 = I2C Master mode, clock = FOSC / (4 * (SSPADD+1))
		movwf	SSP2CON1

;set-up I2C speed
		movlw	B'00111111'
		movwf	SSP2ADD
		clrf	SSP2STAT

		clrf	BSR		;select BANK 0

		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------



;-----------------------------------------------------------------------------
i2c_start	bsf	BSR,D'002'		;select register BANK 4
		bsf	SSP2CON2, SEN
		bcf	BSR,D'002'		;select register BANK 0
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_stop	bsf	BSR,D'002'
		bsf	SSP2CON2, PEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_ack		bsf	BSR,D'002'
		bcf	SSP2CON2, ACKDT		;ACK
		bsf	SSP2CON2, ACKEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
i2c_not_ack	bsf	BSR,D'002'
		bsf	SSP2CON2, ACKDT		;NOT ACK
		bsf	SSP2CON2, ACKEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
;output: received byte is in W register
;
i2c_receive	bsf	BSR,D'002'
		bsf	SSP2CON2, RCEN
		bcf	BSR,D'002'
		call	d55
		bsf	BSR,D'002'
		movf	SSP2BUF,0
		bcf	BSR,D'002'

		movwf	v_byte
		return
;-----------------------------------------------------------------------------
;input: transmitted byte have to be in W register
;
i2c_send	bsf	BSR,D'002'
		movwf	SSP2BUF
		bcf	BSR,D'002'
		call	d55			;wait a while
		return
;-----------------------------------------------------------------------------
i2c_restart	bsf	BSR,D'002'
		bsf	SSP2CON2, RSEN
		bcf	BSR,D'002'
		call	d55
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
d55		movlw	D'255'
		movwf	TIMER2
d55_loop	decfsz	TIMER2,F
		goto	d55_loop
		return
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;-----------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;One Wire Software Routines
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
wait_007p5us	nop			;0,5us
		nop			;0,5us
		nop			;0,5us
		nop			;0,5us
		nop			;0,5us

		nop			;0,5us
		nop			;0,5us
		nop			;0,5us
		nop			;0,5us
		nop			;0,5us

		nop			;0,5us
		nop			;0,5us
		nop			;0,5us

		return			;1,0us
;------------------------------------------------------------------------------
wait_500us	movlw	D'050'
		movwf	TIMER1

w5mn		call	wait_007p5us	;1,0us
		decfsz	TIMER1,1	;0,5us
		goto	w5mn		;1,0us

		return
;------------------------------------------------------------------------------
wait_430us	movlw	D'043'
		movwf	TIMER1

w4mn		call	wait_007p5us
		decfsz	TIMER1,1
		goto	w4mn

		return
;------------------------------------------------------------------------------
wait_070us	movlw	D'007'
		movwf	TIMER1

w0mn		call	wait_007p5us
		decfsz	TIMER1,1
		goto	w0mn

		return
;------------------------------------------------------------------------------
wait_060us	movlw	D'006'
		movwf	TIMER1

w02mn		call	wait_007p5us
		decfsz	TIMER1,1
		goto	w02mn

		return
;------------------------------------------------------------------------------
wait_050us	movlw	D'005'
		movwf	TIMER1

w03mn		call	wait_007p5us
		decfsz	TIMER1,1
		goto	w03mn

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ow_rst
ow_reset
sow_reset	bcf	SOW_in
		bsf	BSR,D'000'
		bsf	SOW_out
		nop
		bcf	SOW_out			;OW := low
		clrf	BSR
		call	wait_500us
		bsf	BSR,D'000'
		bsf	SOW_out			;OW := high
		clrf	BSR
		call	wait_070us

		btfss	SOW_in
		nop				;here we got Presence Pulse

		call	wait_430us
		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ow_write
sow_write	movlw	D'008'
		movwf	TIMER2

		bcf	SOW_in
sow_write_loop	bsf	BSR,D'000'
		bcf	SOW_out
		nop
		nop
		nop

		nop
		nop
		clrf	BSR

		rrf	ow_buffer,1
		bsf	BSR,D'000'
		btfsc	STATUS,C
		bsf	SOW_out

		clrf	BSR
		call	wait_060us

		bsf	BSR,D'000'
		bsf	SOW_out
		nop
		nop
		nop
		clrf	BSR
		decfsz	TIMER2,1
		goto	sow_write_loop

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
ow_read
sow_read	movlw	D'008'
		movwf	TIMER2

		bcf	SOW_in
sow_read_loop	bsf	BSR,D'000'
		bcf	SOW_out
		nop
		nop
		nop
		nop
		nop
		nop

		nop
		nop
		nop
		nop
		nop
		nop

		bsf	SOW_out
		nop
		nop
		nop
		nop

		nop
		nop
		nop
		clrf	BSR

		clrf	TIMER1
		btfsc	SOW_in
		bsf	TIMER1,D'000'
		movf	TIMER1,0
		addlw	D'255'
		rrf	ow_buffer,1
		call	wait_050us

		decfsz	TIMER2,1
		goto	sow_read_loop

		return
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;------------------------------------------------------------------------------
;xxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
		END